#ifndef lint
static char SCCSid[] = "@(#) ./dgraph/x11grf.c 12/01/93";
#endif

#include "tools.h"
#ifndef TOOLSNOX11
#include "dgraph/dgraph.h"
#include "xtools/basex11.h"
#include "xtools/baseclr.h"
#include "xtools/axis/axis.h"
	
/* X11 routines */
typedef struct {
    XBWindow   *win;
    XBAxisData *ad;
    XBFont     *font;
    double     lastx, lasty;
    } GRXctx;

extern void GRXDrawLine(), GRXAddLine(), GRXDrawLines(),
	    GRXStartPage(), GRXEndPage(), GRXDrawRectangle(),
	    GRXDrawText(), GRXClose(), GRXDrawAxis(),
            GRXSetLimits();

/* #define DEBUG */
#ifdef DEBUG
#define DBGX(here) {fprintf( stdout, "At:%s\n", here ); fflush( stdout );}
#else
#define DBGX(here)
#endif

GRctx *GRXCreate( display, window_name, w, h, x, y )
char *display, *window_name;
int  w, h, x, y;
{
GRctx    *new;
GRXctx *cctx;

new       = NEW(GRctx); CHKPTRN(new);

new->Dev  = cctx = NEW(GRXctx);    CHKPTRN(new->Dev);
cctx->win = XBWinCreate();         CHKPTRN(cctx->win);

XBQuickWindow( cctx->win, display, window_name, x, y, w, h );
cctx->ad   = 0;
cctx->font = XBFontFixed( cctx->win, 7, 13 );
cctx->lastx = 0.0;
cctx->lasty = 0.0;

new->use_color     = 0;
new->is_interactive= 1;
new->DrawAxis      = GRXDrawAxis;
new->SetLimits     = GRXSetLimits;
new->DrawLine      = GRXDrawLine;
new->AddLine       = GRXAddLine;
new->DrawLines     = GRXDrawLines;
new->StartPage     = GRXStartPage;
new->EndPage       = GRXEndPage;
new->DrawRectangle = GRXDrawRectangle;
new->DrawText      = GRXDrawText;
new->Close         = GRXClose;
	
return new;
}

void GRiXConvertToLocal( cctx, x, xi, y, yi )
GRXctx *cctx;
double x, y;
int    *xi, *yi;
{
XBAxisData *ad = cctx->ad;

XBAConvertToLocal( ad, x, xi, y, yi );
}


void GRiSetColorFromStyle( cctx, color )
GRXctx *cctx;
int    color;
{
XBWindow *win = cctx->ad->win;
int      icolor;

switch (color) {
    case GRBLACK: icolor = BLACK;  break;
    case GRWHITE: icolor = WHITE;  break;
    case GRRED:   icolor = RED;    break;
    case GRBLUE:  icolor = BLUE;   break;
    case GRGREEN: icolor = GREEN;  break;
    case GRYELLOW:icolor = YELLOW; break;
    default:      icolor = BLACK;  break;
    }
DBGX("SetColorFromStyle")
XSetForeground( win->disp, win->gc.set, win->cmapping[icolor] );
}

/* General routines */
void GRXClose( gctx )
GRctx *gctx;
{
GRXctx *cctx = (GRXctx *)gctx->Dev;

DBGX("Close")
XBWinDestroy( cctx->win );
if (cctx->ad) 
    XBADestroyAxis( cctx->ad );
}

/* We let C.It manage the coordinate system */    
void GRXDrawLine( gctx, x1, y1, x2, y2, style )
GRctx   *gctx;
double  x1, y1, x2, y2;
GRStyle *style;
{
GRXctx *cctx = (GRXctx *)gctx->Dev;	

GRiSetColorFromStyle( cctx, style->color );
DBGX("DrawLine")
XBADrawLine( cctx->ad, x1, y1, x2, y2 );
}

void GRXAddLine( gctx, x1, y1 )
GRctx  *gctx;
double x1, y1;
{
GRXctx *cctx = (GRXctx *)gctx->Dev;	

GRiSetColorFromStyle( cctx, GRBLACK );
DBGX("AddLine")
XBADrawLine( cctx->ad, cctx->lastx, cctx->lasty, x1, y1 );
cctx->lastx = x1;
cctx->lasty = y1;
}

void GRXDrawLines( gctx, n, x, y, style )
GRctx   *gctx;
int     n;
double  *x, *y;
GRStyle *style;
{
GRXctx *cctx = (GRXctx *)gctx->Dev;
int      i;

DBGX("DrawLines")
GRiSetColorFromStyle( cctx, style->color );
for (i=0; i<n-1; i++) {
    XBADrawLine( cctx->ad, x[i], y[i], x[i+1], y[i+1] );
    }
}

void GRXStartPage( gctx )
GRctx  *gctx;
{
GRXctx   *cctx = (GRXctx *)gctx->Dev;
XBWindow *XBWin = cctx->win;

/* We need a routine to clear the entire window... */
DBGX("StartPage")
XBClearWindow( cctx->win, 0, 0, XBWin->w, XBWin->h );
}

void GRXEndPage( gctx )
GRctx  *gctx;
{
GRXctx *cctx = (GRXctx *)gctx->Dev;

DBGX("EndPage")
XBFlush( cctx->win );
}

void GRXDrawRectangle( gctx, x1, x2, y1, y2, style )
GRctx   *gctx;
double  x1, x2, y1, y2;
GRStyle *style;
{
GRXctx *cctx = (GRXctx *)gctx->Dev;

DBGX("DrawRectangle")
GRiSetColorFromStyle( cctx, style->color );
XBADrawLine( cctx->ad, x1, y1, x1, y2 );
XBADrawLine( cctx->ad, x1, y2, x2, y2 );
XBADrawLine( cctx->ad, x2, y2, x2, y1 );
XBADrawLine( cctx->ad, x2, y1, x1, y1 );
}

void GRXDrawText( gctx, x, y, str, size, just )
GRctx  *gctx;
double x, y;
char   *str;
{
GRXctx *cctx = (GRXctx *)gctx->Dev;
int    xi, yi;

DBGX("DrawText")
GRiSetColorFromStyle( cctx, GRBLACK );
/* Need to adjust for justification/size */
XBAConvertToLocal( cctx->ad, x, &xi, y, &yi );
XBDrawText( cctx->win, cctx->font, xi, yi, str );
}

void GRXSetLimits( gctx, xmin, xmax, ymin, ymax )
GRctx  *gctx;
double xmin, xmax, ymin, ymax;
{
GRXctx *cctx = (GRXctx *)gctx->Dev;

DBGX("SetLimts")
gctx->xmin = xmin;
gctx->xmax = xmax;
gctx->ymin = ymin;
gctx->ymax = ymax;

if (!cctx->ad) 
    cctx->ad = XBAInitAxis( cctx->win, cctx->font );
XBASetLimits( cctx->ad, xmin, xmax, ymin, ymax );
}

void GRXDrawAxis( gctx, flag )
GRctx  *gctx;
int    flag;
{
GRXctx *cctx = (GRXctx *)gctx->Dev;

DBGX("DrawAxis")
if (!cctx->ad) 
    cctx->ad = XBAInitAxis( cctx->win, cctx->font );
if (flag) {
    GRiSetColorFromStyle( cctx, GRBLACK );
    XBADrawAxis( cctx->ad );
    }
}

#endif
