#ifndef lint
static char SCCSid[] = "@(#) ./doc/outtex.c 12/01/93";
#endif

#include <string.h>
#include <stdio.h>
#include <ctype.h>
#include "tools.h"
#include "doc/doc.h"

/*
    This file contains routines to generate LaTeX output for the structured
    documentation commands
 */

static int InVerbatim = 0; /* ????? */

void OutputCharLaTeX( fout, c )
FILE *fout;
char c;
{
if (c == '\\') 
    fputs( "\\\\", fout );
else {
    if (!InVerbatim) {
	if (c == '<')
	    fputs( "$<$", fout );
	else if (c == '>') 
	    fputs( "$>$", fout );
	else if (c == '=')
	    fputs( "$=$", fout );
	else if (c == '^')
	    fputs( "{\\tt \\char`\\^}", fout );
	else if (c == '_')
	    fputs( "{\\tt \\char`\\_}", fout );
	else if (c == '#')
	    fputs( "{\\tt \\char`\\#}", fout );
	else if (c == '&')
	    fputs( "\\&", fout );
	else if (c == '$')
	    fputs( "\\$", fout );
	else if (c == '%')
	    fputs( "\\%", fout );
	else
	    putc( c, fout );
	}
    else
	putc( c, fout );
    }
}

void OutputRawCharLaTeX( fout, c )
FILE *fout;
char c;
{
putc( c, fout );
}

void OutputStringLaTeX( fout, s )
FILE *fout;
char *s;
{
char *p, c, *rstr = 0;

/* look for special characters */
p = s;
while (*p) {
    c = *p;
    rstr = 0;
    if (c == '\\') 
	rstr = "\\\\";
    if (!InVerbatim) {
	if (c == '<')
	    rstr = "$<$";
	else if (c == '>') 
	    rstr = "$>$";
	else if (c == '=')
	    rstr = "$=$";
	else if (c == '^')
	    rstr = "{\\tt \\char`\\^}";
	else if (c == '_')
	    rstr = "{\\tt \\char`\\_}";
	else if (c == '#')
	    rstr = "{\\tt \\char`\\#}";
	else if (c == '&')
	    rstr = "\\&";
	else if (c == '%')
	    rstr = "\\%";
	else if (c == '$')
	    rstr = "\\$";
	}

    if (rstr) {
        *p = '\0';
        fputs( s, fout );
        fputs( rstr, fout );
	s = p + 1;
        }
    p++;
    }
if (*s) 
    fputs( s, fout );
}

void OutputBlankLaTeX( fout )
FILE *fout;
{
fputs( "\\ ", fout );
}

void OutputLocationLaTeX( fout, s )
FILE *fout;
char *s;
{
fprintf( fout, "\n\\location{%s}\n", s );
}

void OutputTitleLaTeX( fout, name, level, date )
FILE *fout;
char *name, *level, *date;
{
char bname[256];
strcpy( bname, name );
/* We have to be prepared to correct the name if it contains a
   reserved character */
fputs( "\\mantitle{", fout );
OutputStringLaTeX( fout, name );
fprintf( fout, "}{%s}{%s}\n\\manname{", level, date, name );
OutputStringLaTeX( fout, bname );
fputs( "}\n", fout );
}

void OutputSectionLaTeX( fout, name )
FILE *fout;
char *name;
{
fprintf( fout, "\\subhead{%s}\n", name );
}

void OutputLineBreakLaTeX( lastnl, fout )
int  lastnl;
FILE *fout;
{
fputs( "\\nextline\n", fout );
}

void OutputArgBeginLaTeX( fout )
FILE *fout;
{
fputs( "\\startarg{", fout );
}

void OutputArgDefnLaTeX( fin, fout )
FILE *fin, *fout;
{
int c;

SkipWhite( fin );
fputs( "}{", fout );
c = getc( fin );
if (c != '-') 
    ungetc( (char)c, fin );
else
    SkipWhite( fin );
}

void OutputArgEndLaTeX( fout )
FILE *fout;
{
fputs( "}\n", fout );
}

void OutputVerbatimBeginLaTeX( fout )
FILE *fout;
{
fputs( "\\startvb\\begin{verbatim}\n", fout );
InVerbatim = 1;
}

void OutputVerbatimEndLaTeX( fout )
FILE *fout;
{
fputs( "\\end{verbatim}\n\\endvb", fout );
InVerbatim = 0;
}

void OutputEndPageLaTeX( fout )
FILE *fout;
{
fputs( "\\endmanpage\n", fout );
}

DocOutput *CreateOutputLaTeX( )
{
DocOutput *new;

new = NEW( DocOutput );
new->outchar          = OutputCharLaTeX;
new->outraw           = OutputRawCharLaTeX;
new->outstring        = OutputStringLaTeX;
new->outblank         = OutputBlankLaTeX;
new->outlocation      = OutputLocationLaTeX;
new->outtitle         = OutputTitleLaTeX;
new->outsection       = OutputSectionLaTeX;
new->outlinebreak     = OutputLineBreakLaTeX;
new->outargbegin      = OutputArgBeginLaTeX;
new->outargdefn       = OutputArgDefnLaTeX;
new->outargend        = OutputArgEndLaTeX;
new->outverbatimbegin = OutputVerbatimBeginLaTeX;
new->outverbatimend   = OutputVerbatimEndLaTeX;
new->outendpage       = OutputEndPageLaTeX;

return new;
}
